==> recommenderlab_0.1-4.tar.gz: recommenderlab/R/realRatingMatrix.R <==
## realRatingMatrix

## coercion
setAs("matrix", "realRatingMatrix",
	function(from) new("realRatingMatrix", 
		data = dropNA(from)))

setAs("realRatingMatrix", "matrix",
	function(from) dropNA2matrix(from@data))

## from a data.frame with columns user, item, rating
## this perserves 0s
setAs("data.frame", "realRatingMatrix", function(from) {
		user	<- from[,1]
		item	<- from[,2]
		if(ncol(from)>=3) rating <- as.numeric(from[,3])
		else rating <- rep(1, length(item))

		i <- factor(user)
		j <- factor(item)

		dgT <- new("dgTMatrix", i = as.integer(i)-1L, j = as.integer(j)-1L, 
			x = rating,
			Dim = c(length(levels(i)), length(levels(j))),
			Dimnames = list(levels(i),levels(j)))
	
		new("realRatingMatrix", data = as(dgT, "dgCMatrix"))
	})

setAs("realRatingMatrix", "dgCMatrix",
	function(from) from@data)

setAs("realRatingMatrix", "dgTMatrix",
	function(from) as(from@data, "dgTMatrix"))

setAs("realRatingMatrix", "ngCMatrix",
	function(from) as(from@data, "ngCMatrix"))

setMethod("getList", signature(from = "realRatingMatrix"),
	function(from, decode = TRUE, ratings = TRUE,...) {
		trip <- as(from@data, "dgTMatrix")
		lst <- split(trip@j+1L, factor(trip@i, 
				levels=0:(nrow(trip)-1L)), drop=FALSE)
		rts <- split(trip@x, factor(trip@i, 
				levels=0:(nrow(trip)-1L)), drop=FALSE)

		if(decode) lst <- lapply(lst, function(y) colnames(from)[y])
		else names(lst) <- NULL

		if(ratings) {
			for(i in 1:length(rts)) {
				names(rts[[i]]) <- lst[[i]]
			}
		}else{
			rts <- lst
		}
		
		names(rts) <- rownames(from)
		rts
	})


## binarize
setMethod("binarize", signature(x = "realRatingMatrix"),
	function(x, minRating, ...){
		x <- x@data
		x@x <- as.numeric(x@x>=minRating)
		x <- drop0(x)
		if(is.null(colnames(x))) colnames(x) <- 1:ncol(x)
		x <- new("itemMatrix", data = t(as(x, "ngCMatrix")), 
			itemInfo = data.frame(labels=colnames(x)))
		new("binaryRatingMatrix", data = x)
	})


## ratings
setMethod("getRatings", signature(x = "realRatingMatrix"),
	function(x) x@data@x )


setMethod("removeKnownRatings", signature(x = "realRatingMatrix"),
	function(x, known, replicate=FALSE) {
	    if(!is(known, "ratingMatrix")) stop("known needs to be a ratingMatrix!")
	    
	    if(replicate && nrow(x)==1) {
		x@data <- as(t(crossprod(x@data, 
					t(rep(1, nrow(known))))), "dgCMatrix")
		rownames(x) <- rownames(known)
	    }

	    if(nrow(x) != nrow(known))
		stop("removeKnownRatings: Number of rows in x and known do not match!")

	    ## FIXME: make sparse
	    xm <- as(x, "matrix")
	    xm[as(as(known,"ngCMatrix"), "matrix")] <- NA
	    x@data <- dropNA(xm)
	    x
	})


## compute standard deviation
.dgC2list <- function(x, row=TRUE) {
 if(row) x <- t(x)   
 lapply(2:length(x@p), FUN = function(i) {
	     if(x@p[i-1L]==x@p[i]) numeric(0)
	     else x@x[(x@p[i-1L]+1L):x@p[i]]
	 })
}

setMethod("rowSds", signature(x = "realRatingMatrix"),
	function(x, ...) {
	    s <- sapply(.dgC2list(x@data, row=TRUE), sd)
	    names(s) <- rownames(x)
	    s
	})


setMethod("colSds", signature(x = "realRatingMatrix"),
	function(x, ...) {
	    s <- sapply(.dgC2list(x@data, row=FALSE), sd)
	    names(s) <- colnames(x)
	    s
	})



## create test data
setMethod(".splitKnownUnknown", signature(data="realRatingMatrix"),
	function(data, given) {

		## given might of length one or length(data)
		if(length(given)==1) given <- rep(given, nrow(data))

		## we create a logical mask via a triplet Matrix
		trip <- as(data, "dgTMatrix")
		items <- lapply(0:(nrow(data)-1), 
			function(i) which(trip@i == i))

		take <-  unlist(lapply(1:length(items), 
				function(i) sample(items[[i]],given[i])))

		tripUnknown <- trip
		tripUnknown@x <- tripUnknown@x[-take]
		tripUnknown@i <- tripUnknown@i[-take]
		tripUnknown@j <- tripUnknown@j[-take]
		tripKnown <- trip
		tripKnown@x <- tripKnown@x[take]
		tripKnown@i <- tripKnown@i[take]
		tripKnown@j <- tripKnown@j[take]

		known <- new("realRatingMatrix", 
			data = as(tripKnown, "dgCMatrix"))
		unknown <- new("realRatingMatrix", 
			data = as(tripUnknown, "dgCMatrix"))

		list(
			known = known,
			unknown = unknown
		)
	})

==> recommenderlab_0.1-6.tar.gz: recommenderlab/R/realRatingMatrix.R <==
## realRatingMatrix

## coercion
setAs("matrix", "realRatingMatrix",
	function(from) new("realRatingMatrix", 
		data = dropNA(from)))

setAs("realRatingMatrix", "matrix",
	function(from) dropNA2matrix(from@data))

## from a data.frame with columns user, item, rating
## this perserves 0s
setAs("data.frame", "realRatingMatrix", function(from) {
		user	<- from[,1]
		item	<- from[,2]
		if(ncol(from)>=3) rating <- as.numeric(from[,3])
		else rating <- rep(1, length(item))

		i <- factor(user)
		j <- factor(item)

		dgT <- new("dgTMatrix", i = as.integer(i)-1L, j = as.integer(j)-1L, 
			x = rating,
			Dim = c(length(levels(i)), length(levels(j))),
			Dimnames = list(levels(i),levels(j)))
	
		new("realRatingMatrix", data = as(dgT, "dgCMatrix"))
	})

setAs("realRatingMatrix", "dgCMatrix",
	function(from) from@data)

setAs("realRatingMatrix", "dgTMatrix",
	function(from) as(from@data, "dgTMatrix"))

setAs("realRatingMatrix", "ngCMatrix",
	function(from) as(from@data, "ngCMatrix"))

setMethod("getList", signature(from = "realRatingMatrix"),
	function(from, decode = TRUE, ratings = TRUE,...) {
		trip <- as(from@data, "dgTMatrix")
		lst <- split(trip@j+1L, factor(trip@i, 
				levels=0:(nrow(trip)-1L)), drop=FALSE)
		rts <- split(trip@x, factor(trip@i, 
				levels=0:(nrow(trip)-1L)), drop=FALSE)

		if(decode) lst <- lapply(lst, function(y) colnames(from)[y])
		else names(lst) <- NULL

		if(ratings) {
			for(i in 1:length(rts)) {
				names(rts[[i]]) <- lst[[i]]
			}
		}else{
			rts <- lst
		}
		
		names(rts) <- rownames(from)
		rts
	})


## binarize
setMethod("binarize", signature(x = "realRatingMatrix"),
	function(x, minRating, ...){
		x <- x@data
		x@x <- as.numeric(x@x>=minRating)
		x <- drop0(x)
		if(is.null(colnames(x))) colnames(x) <- 1:ncol(x)
		x <- as(t(x), "ngCMatrix")
    
    new("binaryRatingMatrix", data = as(x, "itemMatrix"))
	})


## ratings
setMethod("getRatings", signature(x = "realRatingMatrix"),
	function(x) x@data@x )


setMethod("removeKnownRatings", signature(x = "realRatingMatrix"),
	function(x, known, replicate=FALSE) {
	    if(!is(known, "ratingMatrix")) stop("known needs to be a ratingMatrix!")
	    
	    if(replicate && nrow(x)==1) {
		x@data <- as(t(crossprod(x@data, 
					t(rep(1, nrow(known))))), "dgCMatrix")
		rownames(x) <- rownames(known)
	    }

	    if(nrow(x) != nrow(known))
		stop("removeKnownRatings: Number of rows in x and known do not match!")

	    ## FIXME: make sparse
	    xm <- as(x, "matrix")
	    xm[as(as(known,"ngCMatrix"), "matrix")] <- NA
	    x@data <- dropNA(xm)
	    x
	})


## compute standard deviation
.dgC2list <- function(x, row=TRUE) {
 if(row) x <- t(x)   
 lapply(2:length(x@p), FUN = function(i) {
	     if(x@p[i-1L]==x@p[i]) numeric(0)
	     else x@x[(x@p[i-1L]+1L):x@p[i]]
	 })
}

setMethod("rowSds", signature(x = "realRatingMatrix"),
	function(x, ...) {
	    s <- sapply(.dgC2list(x@data, row=TRUE), sd)
	    names(s) <- rownames(x)
	    s
	})


setMethod("colSds", signature(x = "realRatingMatrix"),
	function(x, ...) {
	    s <- sapply(.dgC2list(x@data, row=FALSE), sd)
	    names(s) <- colnames(x)
	    s
	})



## create test data
setMethod(".splitKnownUnknown", signature(data="realRatingMatrix"),
	function(data, given) {

		## given might of length one or length(data)
		if(length(given)==1) given <- rep(given, nrow(data))

		## we create a logical mask via a triplet Matrix
		trip <- as(data, "dgTMatrix")
		items <- lapply(0:(nrow(data)-1), 
			function(i) which(trip@i == i))

		take <-  unlist(lapply(1:length(items), 
				function(i) sample(items[[i]],given[i])))

		tripUnknown <- trip
		tripUnknown@x <- tripUnknown@x[-take]
		tripUnknown@i <- tripUnknown@i[-take]
		tripUnknown@j <- tripUnknown@j[-take]
		tripKnown <- trip
		tripKnown@x <- tripKnown@x[take]
		tripKnown@i <- tripKnown@i[take]
		tripKnown@j <- tripKnown@j[take]

		known <- new("realRatingMatrix", 
			data = as(tripKnown, "dgCMatrix"))
		unknown <- new("realRatingMatrix", 
			data = as(tripUnknown, "dgCMatrix"))

		list(
			known = known,
			unknown = unknown
		)
	})
